# simple tests

import pytest
from scrall.parse.parser import ScrallParser
from scrall.parse.visitor import (Execution_Unit_a, Signal_a, Signal_Dest_a, N_a, INST_a, PATH_a, R_a,
    Seq_Statement_Set_a, Supplied_Parameter_a, External_Signal_a, Decision_a, Comp_Statement_Set_a)

actions = [
    ("Time to close ->* me",
     Execution_Unit_a(
         statement_set=Seq_Statement_Set_a(
             input_tokens=[],
             statement=Signal_a(
                 event='Time to close',
                 supplied_params=[],
                 dest=Signal_Dest_a(target_iset=N_a(name='me'), assigner_dest=None, delay=0, cancel=True)),
             block=None),
         output_token=None)
     ),
    ("Goto floor( Dest floor: my level) -> ~",
     Execution_Unit_a(
         statement_set=Seq_Statement_Set_a(
             input_tokens=[],
             statement=External_Signal_a(
                 event='Goto floor',
                 supplied_params=[
                     Supplied_Parameter_a(pname='Dest floor', sval=N_a(name='my level'))
                 ]), block=None),
         output_token=None)
     ),
    ("Change requested -> me",
        Execution_Unit_a(
            statement_set=Seq_Statement_Set_a(
                input_tokens=[],
                statement=Signal_a(
                    event='Change requested',
                    supplied_params=[],
                    dest=Signal_Dest_a(target_iset=N_a(name='me'), assigner_dest=None, delay=0, cancel=False)),
                block=None),
            output_token=None)
     ),
    ("Ready to go -> /R53/Transfer",
        Execution_Unit_a(
            statement_set=Seq_Statement_Set_a(
                input_tokens=[],
                statement=Signal_a(
                    event='Ready to go',
                    supplied_params=[],
                    dest=Signal_Dest_a(target_iset=INST_a(
                        components=[PATH_a(hops=[R_a(rnum='R53'), N_a(name='Transfer')])]
                    ), assigner_dest=None, delay=0, cancel=False)
                ),
                block=None),
            output_token=None)
     ),
    ("destination aslev?\n"
     "    Transfer created -> : No destination -> me",
     Execution_Unit_a(
         statement_set=Seq_Statement_Set_a(
             input_tokens=[],
             statement=Decision_a(
                 input=N_a(name='destination aslev'),
                 true_result=Comp_Statement_Set_a(
                     statement=Signal_a(
                         event='Transfer created',
                         supplied_params=[], dest=None), block=None),
                 false_result=Comp_Statement_Set_a(
                     statement=Signal_a(
                         event='No destination',
                         supplied_params=[],
                         dest=Signal_Dest_a(
                             target_iset=N_a(name='me'), assigner_dest=None, delay=0, cancel=False
                         )), block=None
                 )), block=None), output_token=None)
     ),
]


@pytest.mark.parametrize("text, expected", actions)
def test_signal_action(text, expected):
    parse = ScrallParser.parse_text(scrall_text=text, debug=False)[0]
    print(parse)
    assert parse[0] == expected



""" test_decision_scalar_expr.py -- Ensure scalar expression works as true result in a decision"""

# This verifies that we don't have a regression on a bug fix where scalar_expr true results were eating the colon
# and failing the parse

import pytest
from scrall.parse.parser import ScrallParser
from scrall.parse.visitor import *

actions = [
    ("fwd dest? =>> fwd dest : {\n"
     "    rev dest .= Ping( dir: Travel direction.opposite )\n"
     "    !rev dest? Travel direction.toggle : =>> rev dest\n"
     "}",
        Execution_Unit_a(
            statement_set=Seq_Statement_Set_a(
                input_tokens=[],
                statement=Decision_a(
                    input=N_a(name='fwd dest'),
                    true_result=Comp_Statement_Set_a(
                        statement=Output_Flow_a(output=N_a(name='fwd dest')), block=None),
                    false_result=Comp_Statement_Set_a(
                        statement=None, block=[
                            Execution_Unit_a(
                                statement_set=Seq_Statement_Set_a(
                                    input_tokens=[],
                                    statement=Inst_Assignment_a(
                                    lhs=Flow_Output_a(name=N_a(name='rev dest'), exp_type=None), card='1',
                                    rhs=INST_a(components=
                                        [N_a(name='Ping'),
                                        Criteria_Selection_a(
                                        card='ALL',
                                        criteria=BOOL_a(op='==',
                                        operands=[N_a(name='dir'),
                                            INST_PROJ_a(
                                                qty=False,
                                                iset=N_a(name='Travel direction'),
                                                projection=Projection_a(
                                                expand=None,
                                                attrs=[N_a(name='opposite')]), op_chain=None)
                                                  ]))]), X=(31, 81)), block=None),
                                output_token=None),
                            Execution_Unit_a(
                                statement_set=Seq_Statement_Set_a(
                                    input_tokens=[],
                                    statement=Decision_a(
                                        input=BOOL_a(op='NOT', operands=N_a(name='rev dest')),
                                        true_result=Comp_Statement_Set_a(
                                            statement=Call_a(
                                                call=N_a(name='Travel direction'),
                                                op_chain=Op_chain_a(components=[N_a(name='toggle')])), block=None),
                                        false_result=Comp_Statement_Set_a(
                                            statement=Output_Flow_a(
                                                output=N_a(name='rev dest')), block=None)), block=None),
                                output_token=None)
                        ]
                    )
                ), block=None
            ), output_token=None),
     )
]

@pytest.mark.parametrize("text, expected", actions)
def test_decision_scalar_true_result(text, expected):
    parse = ScrallParser.parse_text(scrall_text=text, debug=False)[0]
    print(parse)
    assert parse[0] == expected



""" test_calls.py - Test method calls and calls to an external service """

import pytest
from scrall.parse.parser import ScrallParser
from scrall.parse.visitor import *

actions = [
    ("~.Goto floor( Dest floor: ^new dest )",
    Execution_Unit_a(
        statement_set=Seq_Statement_Set_a(
            input_tokens=[],
            statement=Call_a(
                call=INST_a(
                    components=[
                        Op_a(owner='_external', op_name='Goto floor',
                             supplied_params=[
                                 Supplied_Parameter_a(
                                     pname='Dest floor', sval=IN_a(name='new dest')
                                 )
                             ])]), op_chain=None), block=None), output_token=None)
     ),
    #  Execution_Unit_a(
    #      statement_set=Seq_Statement_Set_a(
    #          input_tokens=[],
    #          statement=Call_a(
    #              call=None,
    #              op_chain=Op_chain_a(
    #                  components=[
    #                      Scalar_op_a(
    #                          name=N_a(name='Goto floor'),
    #                          supplied_params=[
    #                              Supplied_Parameter_a(
    #                                  pname='Dest floor',
    #                                  sval=IN_a(name='new dest')
    #                              )
    #                          ]
    #                      )
    #                  ]
    #              )
    #          ), block=None),
    #      output_token=None)
    #  ),
    (".Ping( dir: Travel direction.opposite )",
    Execution_Unit_a(
        statement_set=Seq_Statement_Set_a(
            input_tokens=[],
            statement=Call_a(
                call=INST_a(
                    components=[
                        Op_a(
                            owner='_implicit',
                            op_name='Ping',
                            supplied_params=[
                                Supplied_Parameter_a(
                                    pname='dir',
                                    sval=INST_PROJ_a(
                                        qty=False,
                                        iset=N_a(name='Travel direction'),
                                        projection=Projection_a(
                                            expand=None, attrs=[N_a(name='opposite')]
                                        ),
                                        op_chain=None
                                    )
                                )
                            ]
                        )
                    ]
                ),
                op_chain=None),
            block=None),
        output_token=None)
     ),
]

@pytest.mark.parametrize("text, expected", actions)
def test_call_action(text, expected):
    parse = ScrallParser.parse_text(scrall_text=text, debug=False)[0]
    print(parse)
    assert parse[0] == expected
